#ifndef HSV_COLOR_HPP
#define HSV_COLOR_HPP

#include <cmath>
#include <iostream>
#include "rgbcolor.hpp"

RGBColor HSVtoRGB(float h, float s, float v);

struct HSVColor {
   HSVColor() : hue(0), saturation(0), value(0) {
   }
   HSVColor(float hue, float saturation, float value) :
      hue(hue), saturation(saturation), value(value) {
   }
   RGBColor get_rgb() const {
      return HSVtoRGB(hue, saturation, value);
   }
   HSVColor interpolate(const HSVColor& other, float t) {
      return HSVColor(
	 (other.hue <= hue?
	       std::fmod(hue + (360 + other.hue - hue) * t, 360)
	    :
	       hue + (other.hue - hue) * t),
	 saturation + (other.saturation - saturation) * t,
	 value + (other.value - value) * t
      );
   }
   float hue;
   float saturation;
   float value;
};

inline std::ostream& operator<<(std::ostream& out, const HSVColor& color) {
   return out << "(" << color.hue << ", " << color.saturation <<
      ", " << color.value << ")";
}

#endif
